@@ -1821,10 +1821,17 @@ char *cbmimebreak(const char *ptr, int size, CBMAP *attrs, int *sp){
             cbmapput(attrs, "CHARSET", -1, pv, ep - pv, TRUE);
           } else if(cbstrfwimatch(ep, "boundary=")){
             ep += 9;
-            if(ep[0] == '"') ep++;
-            pv = ep;
-            while(ep[0] != '\0' && ep[0] != ' ' && ep[0] != '"' && ep[0] != ';'){
+            if(ep[0] == '"'){
               ep++;
+              pv = ep;
+              while(ep[0] != '\0' && ep[0] != '"'){
+                ep++;
+              }
+            } else {
+              pv = ep;
+              while(ep[0] != '\0' && ep[0] != ' ' && ep[0] != '"' && ep[0] != ';'){
+                ep++;
+              }
             }
             cbmapput(attrs, "BOUNDARY", -1, pv, ep - pv, TRUE);
           }
@@ -1881,7 +1888,7 @@ CBLIST *cbmimeparts(const char *ptr, int size, const char *boundary){
   pv = NULL;
   for(i = 0; i < size; i++){
     if(ptr[i] == '-' && ptr[i+1] == '-' && i + 2 + blen < size &&
-       cbstrfwmatch(ptr + i + 2, boundary)){
+       cbstrfwmatch(ptr + i + 2, boundary) && strchr("\t\n\v\f\r ", ptr[i+2+blen])){
       pv = ptr + i + 2 + blen;
       if(*pv == '\r') pv++;
       if(*pv == '\n') pv++;
@@ -1893,7 +1900,7 @@ CBLIST *cbmimeparts(const char *ptr, int size, const char *boundary){
   if(!pv) return list;
   for(i = 0; i < size; i++){
     if(ptr[i] == '-' && ptr[i+1] == '-' && i + 2 + blen < size &&
-       cbstrfwmatch(ptr + i + 2, boundary)){
+       cbstrfwmatch(ptr + i + 2, boundary) && strchr("\t\n\v\f\r -", ptr[i+2+blen])){
       ep = ptr + i;
       if(ep > ptr && ep[-1] == '\n') ep--;
       if(ep > ptr && ep[-1] == '\r') ep--;
