@@ -55,9 +55,9 @@ void pdperror(const char *name);
 int myrand(void);
 int dowrite(const char *name, int rnum, int bnum, int dnum, int sparse, int lob);
 int doread(const char *name, int wb, int lob);
-int dorcat(const char *name, int rnum, int bnum, int dnum, int pnum, int align);
+int dorcat(const char *name, int rnum, int bnum, int dnum, int pnum, int align, int cb);
 int docombo(const char *name);
-int dowicked(const char *name, int rnum);
+int dowicked(const char *name, int rnum, int cb);
 
 
 /* main routine */
@@ -93,9 +93,9 @@ void usage(void){
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s write [-s] [-lob] name rnum bnum dnum\n", progname);
   fprintf(stderr, "  %s read [-wb] [-lob] name\n", progname);
-  fprintf(stderr, "  %s rcat name rnum bnum dnum pnum align\n", progname);
+  fprintf(stderr, "  %s rcat [-c] name rnum bnum dnum pnum align\n", progname);
   fprintf(stderr, "  %s combo name\n", progname);
-  fprintf(stderr, "  %s wicked name rnum\n", progname);
+  fprintf(stderr, "  %s wicked [-c] name rnum\n", progname);
   fprintf(stderr, "\n");
   exit(1);
 }
@@ -176,7 +176,7 @@ int runread(int argc, char **argv){
 /* parse arguments of rcat command */
 int runrcat(int argc, char **argv){
   char *name, *rstr, *bstr, *dstr, *pstr, *astr;
-  int i, rnum, bnum, dnum, pnum, align, rv;
+  int i, rnum, bnum, dnum, pnum, align, cb, rv;
   name = NULL;
   rstr = NULL;
   bstr = NULL;
@@ -187,9 +187,14 @@ int runrcat(int argc, char **argv){
   bnum = 0;
   pnum = 0;
   align = 0;
+  cb = FALSE;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      usage();
+      if(!strcmp(argv[i], "-c")){
+        cb = TRUE;
+      } else {
+        usage();
+      }
     } else if(!name){
       name = argv[i];
     } else if(!rstr){
@@ -213,7 +218,7 @@ int runrcat(int argc, char **argv){
   pnum = atoi(pstr);
   align = atoi(astr);
   if(rnum < 1 || bnum < 1 || dnum < 1 || pnum < 1) usage();
-  rv = dorcat(name, rnum, bnum, dnum, pnum, align);
+  rv = dorcat(name, rnum, bnum, dnum, pnum, align, cb);
   return rv;
 }
 
@@ -241,13 +246,18 @@ int runcombo(int argc, char **argv){
 /* parse arguments of wicked command */
 int runwicked(int argc, char **argv){
   char *name, *rstr;
-  int i, rnum, rv;
+  int i, rnum, cb, rv;
   name = NULL;
   rstr = NULL;
   rnum = 0;
+  cb = FALSE;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      usage();
+      if(!strcmp(argv[i], "-c")){
+        cb = TRUE;
+      } else {
+        usage();
+      }
     } else if(!name){
       name = argv[i];
     } else if(!rstr){
@@ -259,7 +269,7 @@ int runwicked(int argc, char **argv){
   if(!name || !rstr) usage();
   rnum = atoi(rstr);
   if(rnum < 1) usage();
-  rv = dowicked(name, rnum);
+  rv = dowicked(name, rnum, cb);
   return rv;
 }
 
@@ -295,8 +305,8 @@ int dowrite(const char *name, int rnum, int bnum, int dnum, int sparse, int lob)
   CURIA *curia;
   int i, omode, err, len;
   char buf[RECBUFSIZ];
-  printfflush("<Writing Test>\n  name=%s  rnum=%d  bnum=%d  dnum=%d  lob=%d\n\n",
-              name, rnum, bnum, dnum, lob);
+  printfflush("<Writing Test>\n  name=%s  rnum=%d  bnum=%d  dnum=%d  s=%d  lob=%d\n\n",
+              name, rnum, bnum, dnum, sparse, lob);
   /* open a database */
   omode = CR_OWRITER | CR_OCREAT | CR_OTRUNC | (sparse ? CR_OSPARSE : 0);
   if(!(curia = cropen(name, omode, bnum, dnum))){
@@ -403,12 +413,14 @@ int doread(const char *name, int wb, int lob){
 
 
 /* perform rcat command */
-int dorcat(const char *name, int rnum, int bnum, int dnum, int pnum, int align){
+int dorcat(const char *name, int rnum, int bnum, int dnum, int pnum, int align, int cb){
   CURIA *curia;
-  int i, err, len;
-  char buf[RECBUFSIZ];
-  printfflush("<Random Writing Test>\n  name=%s  rnum=%d  bnum=%d  dnum=%d  pnum=%d  align=%d\n\n",
-              name, rnum, bnum, dnum, pnum, align);
+  CBMAP *map;
+  int i, err, len, ksiz, vsiz, rsiz;
+  const char *kbuf, *vbuf;
+  char buf[RECBUFSIZ], *rbuf;
+  printfflush("<Random Writing Test>\n  name=%s  rnum=%d  bnum=%d  dnum=%d  pnum=%d"
+              "  align=%d  c=%d\n\n", name, rnum, bnum, dnum, pnum, align, cb);
   if(!(curia = cropen(name, CR_OWRITER | CR_OCREAT | CR_OTRUNC, bnum, dnum))){
     pdperror(name);
     return 1;
@@ -418,6 +430,8 @@ int dorcat(const char *name, int rnum, int bnum, int dnum, int pnum, int align){
     crclose(curia);
     return 1;
   }
+  map = NULL;
+  if(cb) map = cbmapopen();
   err = FALSE;
   for(i = 1; i <= rnum; i++){
     len = sprintf(buf, "%08d", myrand() % pnum + 1);
@@ -434,6 +448,27 @@ int dorcat(const char *name, int rnum, int bnum, int dnum, int pnum, int align){
       }
     }
   }
+  if(map){
+    printfflush("Matching records ... ");
+    cbmapiterinit(map);
+    while((kbuf = cbmapiternext(map, &ksiz)) != NULL){
+      vbuf = cbmapget(map, kbuf, ksiz, &vsiz);
+      if(!(rbuf = crget(curia, kbuf, ksiz, 0, -1, &rsiz))){
+        pdperror(name);
+        err = TRUE;
+        break;
+      }
+      if(rsiz != vsiz || memcmp(rbuf, vbuf, rsiz)){
+        fprintf(stderr, "%s: %s: unmatched record\n", progname, name);
+        free(rbuf);
+        err = TRUE;
+        break;
+      }
+      free(rbuf);
+    }
+    cbmapclose(map);
+    if(!err) printfflush("ok\n");
+  }
   if(!crclose(curia)){
     pdperror(name);
     return 1;
@@ -656,9 +691,11 @@ int docombo(const char *name){
 
 
 /* perform wicked command */
-int dowicked(const char *name, int rnum){
+int dowicked(const char *name, int rnum, int cb){
   CURIA *curia;
-  int i, len, err;
+  CBMAP *map;
+  int i, len, err, align, mksiz, mvsiz, rsiz;
+  const char *mkbuf, *mvbuf;
   char buf[RECBUFSIZ], vbuf[RECBUFSIZ], *val;
   printfflush("<Wicked Writing Test>\n  name=%s  rnum=%d\n\n", name, rnum);
   err = FALSE;
@@ -670,20 +707,25 @@ int dowicked(const char *name, int rnum){
     pdperror(name);
     err = TRUE;
   }
+  map = NULL;
+  if(cb) map = cbmapopen();
   for(i = 1; i <= rnum; i++){
     len = sprintf(buf, "%08d", myrand() % rnum + 1);
     switch(myrand() % 16){
     case 0:
       putchar('O');
       if(!crput(curia, buf, len, buf, len, CR_DOVER)) err = TRUE;
+      if(map) cbmapput(map, buf, len, buf, len, TRUE);
       break;
     case 1:
       putchar('K');
       if(!crput(curia, buf, len, buf, len, CR_DKEEP) && dpecode != DP_EKEEP) err = TRUE;
+      if(map) cbmapput(map, buf, len, buf, len, FALSE);
       break;
     case 2:
       putchar('D');
       if(!crout(curia, buf, len) && dpecode != DP_ENOITEM) err = TRUE;
+      if(map) cbmapout(map, buf, len);
       break;
     case 3:
       putchar('G');
@@ -696,9 +738,18 @@ int dowicked(const char *name, int rnum){
     default:
       putchar('C');
       if(!crput(curia, buf, len, buf, len, CR_DCAT)) err = TRUE;
+      if(map) cbmapputcat(map, buf, len, buf, len);
       break;
     }
     if(i % 50 == 0) printfflush(" (%08d)\n", i);
+    if(!err && rnum > 100 && myrand() % (rnum / 100) == 0){
+      if(myrand() % 10 == 0){
+        align = (myrand() % 4 + 1) * -1;
+      } else {
+        align = myrand() % 32;
+      }
+      if(!crsetalign(curia, align)) err = TRUE;
+    }
     if(err){
       pdperror(name);
       break;
@@ -715,6 +766,7 @@ int dowicked(const char *name, int rnum){
       err = TRUE;
       break;
     }
+    if(map) cbmapputcat(map, buf, len, ":", -1);
     putchar(':');
     if(i % 50 == 0) printfflush(" (%08d)\n", i);
   }
@@ -747,6 +799,27 @@ int dowicked(const char *name, int rnum){
     putchar('@');
     if(i % 50 == 0) printfflush(" (%08d)\n", i);
   }
+  if(map){
+    printfflush("Matching records ... ");
+    cbmapiterinit(map);
+    while((mkbuf = cbmapiternext(map, &mksiz)) != NULL){
+      mvbuf = cbmapget(map, mkbuf, mksiz, &mvsiz);
+      if(!(val = crget(curia, mkbuf, mksiz, 0, -1, &rsiz))){
+        pdperror(name);
+        err = TRUE;
+        break;
+      }
+      if(rsiz != mvsiz || memcmp(val, mvbuf, rsiz)){
+        fprintf(stderr, "%s: %s: unmatched record\n", progname, name);
+        free(val);
+        err = TRUE;
+        break;
+      }
+      free(val);
+    }
+    cbmapclose(map);
+    if(!err) printfflush("ok\n");
+  }
   if(!crclose(curia)){
     pdperror(name);
     return 1;
