@@ -1735,7 +1735,7 @@ static int odcacheflushrare(ODEUM *odeum, const char *fname, double ratio){
     odotcb(fname, odeum, otmsg);
   }
   cbmapiterinit(odeum->cachemap);
-  limit = rnum * ratio;
+  limit = (int)(rnum * ratio);
   for(i = 0; i < limit && (kbuf = cbmapiternext(odeum->cachemap, &ksiz)) != NULL; i++){
     vbuf = cbmapget(odeum->cachemap, kbuf, ksiz, &vsiz);
     if(!crput(odeum->indexdb, kbuf, ksiz, vbuf, vsiz, CR_DCAT)){
